package com.pcloud.book.applet.facade;

import com.pcloud.book.applet.biz.AppletRecordServeBiz;
import com.pcloud.book.applet.entity.AppletRecordServe;
import com.pcloud.book.base.exception.BookBizException;
import com.pcloud.common.dto.ResponseDto;
import com.pcloud.common.exceptions.BizException;
import com.pcloud.common.permission.PermissionException;
import com.pcloud.common.utils.SessionUtil;
import com.pcloud.common.utils.cookie.Cookie;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CookieValue;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * (AppletRecordServe)表控制层
 *
 * @author makejava
 * @since 2020-12-09 16:04:33
 */
@RestController("appletRecordServeFacade")
@RequestMapping("appletRecordServe")
public class AppletRecordServeFacade {

    @Autowired
    private AppletRecordServeBiz appletRecordServeBiz;


    @ApiOperation("通过主键查询单条数据")
    @GetMapping("getById")
    public ResponseDto<?> getById(@RequestHeader("token") String token, @RequestParam Long id) throws BizException, PermissionException {
        SessionUtil.getToken4Redis(token);
        return new ResponseDto<>(appletRecordServeBiz.getById(id));
    }

    @ApiOperation("分页查询")
    @GetMapping("getList")
    public ResponseDto<?> getList(@RequestHeader("token") String token,
                                  @RequestParam(value = "currentPage", defaultValue = "0") Integer currentPage,
                                  @RequestParam(value = "numPerPage", defaultValue = "10") Integer numPerPage)
            throws BizException, PermissionException {
        SessionUtil.getToken4Redis(token);
        return new ResponseDto<>(appletRecordServeBiz.getList(currentPage, numPerPage));
    }

    @ApiOperation("新增")
    @PostMapping("insert")
    public ResponseDto<?> insert(@RequestHeader("token") String token, @RequestBody AppletRecordServe appletRecordServe)
            throws BizException, PermissionException {
        SessionUtil.getToken4Redis(token);
        return new ResponseDto<>(appletRecordServeBiz.insert(appletRecordServe));
    }

    @ApiOperation("更新")
    @PostMapping("update")
    public ResponseDto<?> update(@RequestHeader("token") String token, @RequestBody AppletRecordServe appletRecordServe) throws BizException, PermissionException {
        SessionUtil.getToken4Redis(token);
        appletRecordServeBiz.update(appletRecordServe);
        return new ResponseDto<>();
    }

    @ApiOperation("删除")
    @GetMapping("deleteById")
    public ResponseDto<?> deleteById(@RequestHeader("token") String token, @RequestParam Long id) throws BizException, PermissionException {
        SessionUtil.getToken4Redis(token);
        if (null == id) {
            throw BookBizException.PARAM_DELETION;
        }
        appletRecordServeBiz.deleteById(id);
        return new ResponseDto<>();
    }


    @ApiOperation("根据类型获取用户足迹数据")
    @GetMapping("getUserRecordListByType")
    public ResponseDto<?> getUserRecordListByType(@CookieValue("userInfo") String userInfo,
                                                  @RequestParam(value = "recordType",required = false) Integer recordType) throws BizException, PermissionException {
        Long wechatUserId = Cookie.getId(userInfo, Cookie._WECHAT_USER_ID);
        List<AppletRecordServe> appletRecordServeList  = appletRecordServeBiz.getUserRecordListByType(wechatUserId, recordType);
        return new ResponseDto<>(appletRecordServeList);
    }


}